#!/usr/local/bin/python
#
# $Id: //projects/zoe/hedra.py#2 $ $Date: 2002/08/09 $

"""
A sample of regular polyhedra and objects rotating at different rates.
"""

import math
import zoe

from OpenGL.GL   import * # strange, but canonical PyOpenGL import
from OpenGL.GLU  import *
from OpenGL.GLUT import *


class Dodecahedron(zoe.Object):
    def display(self):
        glColor4f(1, 1, 1, 1)
        glPushMatrix()
        glutWireDodecahedron()
        glPopMatrix()

class Cube(zoe.Object):
    def __init__(self, size):
        zoe.Object.__init__(self)
        self.size = size

    def display(self):
        glColor4f(0.5, 0.5, 1, 0.5)
        glPushMatrix()
        glTranslatef(3, 0, 0)
        glutWireCube(self.size)
        glPopMatrix()
        
class Tetrahedron(zoe.Object):
    def display(self):
        glColor4f(0.5, 1, 0.5, 0.5)
        glPushMatrix()
        glTranslatef(3, 0, 0)
        glutWireTetrahedron()
        glPopMatrix()

class Icosahedron(zoe.Object):
    def display(self):
        glColor4f(1, 0, 1, 1)
        glPushMatrix()
        glTranslatef(3, 0, 0)
        glutWireIcosahedron()
        glPopMatrix()

class Octahedron(zoe.Object):
    def display(self):
        glColor4f(1, 1, 0, 1)
        glPushMatrix()
        glTranslatef(3, 0, 0)
        glutWireOctahedron()
        glPopMatrix()

class Plane(zoe.Object):
    def __init__(self, distance, rate):
        self.distance = distance
        self.rate = rate
        self.angle = 0
                 
    def display(self):
        glColor4f(1, 1, 1, 1)
        glPushMatrix()
        glTranslatef(self.distance*math.cos(self.angle),
                     self.distance*math.sin(self.angle), 0)
        degrees = self.angle*zoe.radiansToDegrees
        glRotatef(degrees + 90, 0, 0, 1)
        glBegin(GL_LINE_STRIP)
        glVertex3f(0, 0, 0)
        glVertex3f(1, 0, 0)
        glVertex3f(-1, 0.5, 0)
        glVertex3f(-1, -0.5, 0)
        glVertex3f(1, 0, 0)
        glVertex3f(-1, 0, -0.5)
        glVertex3f(-1, 0, 0)
        glVertex3f(0, 0, 0)
        glEnd()
        glPopMatrix()

    def update(self):
        self.angle += self.rate
        if self.angle >= zoe.twoPi:
            self.angle -= zoe.twoPi


def main():
    engine = zoe.Engine()
    engine.add(zoe.AxesObject())
    engine.add(zoe.GridObject())
    engine.add(zoe.RotatingGroup(0.01, objects=[Dodecahedron()]))
    engine.add(zoe.RotatingGroup(0.02, objects=[Tetrahedron()]))
    engine.add(zoe.RotatingGroup(0.03, objects=[Cube(1.0)]))
    engine.add(zoe.RotatingGroup(0.04, objects=[Icosahedron()]))
    engine.add(zoe.RotatingGroup(0.05, objects=[Octahedron()]))
    engine.add(Plane(5.0, 0.01))
    engine.add(zoe.FrameRateCounter(engine))
    engine.go()

if __name__ == '__main__': main()
