#!/usr/local/bin/python
#
# $Id: //projects/zoe/walk.py#2 $ $Date: 2002/08/09 $

"""
A sample of a number of particles engaging in a random walk.
"""

import random

import la
import zoe


PARITICLES = 100


class WalkParticle(zoe.NewtonianParticle):

    """A particle gets successive random nudges applied to its
    velocity.  It is recycled when it wanders too far."""
    
    EPSILON = 0.01 # the velocity nudge factor
    R_MAX_SQUARED = 10.0**2 # the maximum distance before recycling

    trailLength = 10

    def update(self):
        epsilon = la.Vector(random.uniform(-self.EPSILON, self.EPSILON), \
                            random.uniform(-self.EPSILON, self.EPSILON), \
                            random.uniform(-self.EPSILON, self.EPSILON))
        self.impulse(epsilon)
        zoe.NewtonianParticle.update(self)

    def ok(self):
        return la.Vector(*self.pos).normSquared() <= self.R_MAX_SQUARED


class WalkSystem(zoe.System):

    """Manage the walk system.  All particles start out at rest, at
    the origin."""
    
    def new(self):
        return WalkParticle(la.Vector.ZERO, la.Vector.ZERO)


def main():
    engine = zoe.Engine()
    engine.add(zoe.AxesObject())
    engine.add(zoe.GridObject())
    engine.add(WalkSystem(100))
    engine.add(zoe.FrameRateCounter(engine))
    engine.go()

if __name__ == '__main__': main()
